;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of The Last Fungus: An Amstrad CPC Game 
;;  Copyright (C) 2021 Miguel Perez Tarruella / Antonio Jose Fernandez Belliure / 
;;                     David Martinez Garcia / Ocacho Games (@OcachoGames)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

;;====================================================================
;; GLOBL INCLUDES
;;====================================================================

;;Managers
.include "man/entity.h.s"
.include "man/game.h.s"
.include "man/level_loader.h.s"

;;Resources
.include "resources/players_animations.h.s"
.include "resources/enemies_animations.h.s"

;;Macros
.include "macros/math.h.s"

;;====================================================================
;; NO GRAVITY STATE MACHINE MACROS
;;====================================================================  

.macro SELECT_NO_GRAVITY_PLAYER2_OR_ENEMY_ANIM_MACRO _state, _facing

    ;;We dont need to check if the game is multiplayer cause this only will execute in singleplayer

    ld a, (#m_current_level)
    IS_EQUALS 0
    jr z, label_ghost_easy_'_state'_'_facing

    ;;ghost_difficult_'state:
    ld hl, #ghost_difficult_'_state'_'_facing
    jr update_no_gravity_'_state

    label_ghost_easy_'_state'_'_facing:

    ld hl, #ghost_easy_'_state'_'_facing

    jr update_no_gravity_'_state

.endm 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Parameters
;;  - _state: State we want to define (IDLE, RUN...)
;; Objetive: Define the state for non-gravitables entities
;; Modifies: A, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.macro DEFINE_NO_GRAVITY_ANIM_STATE _state

player_no_gravity_'_state:
    
    ;;We dont need to check what type of entity is bc non-gravitable entities always will be enemies

    ld a, e_facing_right(ix)
    IS_EQUALS 1
    jr z, facing_right_'_state'_p2_or_enemy_no_gravity

    ;;facing_left_'_state':
        SELECT_NO_GRAVITY_PLAYER2_OR_ENEMY_ANIM_MACRO _state, left

    facing_right_'_state'_p2_or_enemy_no_gravity:
        SELECT_NO_GRAVITY_PLAYER2_OR_ENEMY_ANIM_MACRO _state, right

    update_no_gravity_'_state:

    ld e_anim_b1(ix), l
    ld e_anim_b2(ix), h

    ld a, (hl)
    ld e_anim_counter(ix), a
ret

.endm

;;===================================================================
;; STATES
;;===================================================================

DEFINE_NO_GRAVITY_ANIM_STATE idle
DEFINE_NO_GRAVITY_ANIM_STATE run
DEFINE_NO_GRAVITY_ANIM_STATE die

;;====================================================================
;; FUNCTIONS
;;====================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Pre requirements
;;  - IX: should contain the memory direction for the player
;; Objetive: Updates the anim state machine for entities without gravity
;; Modifies: A, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_no_gravity_state_machine_update::

    ;;Check if the entity is set to dying_animation
    IS_ENTITY_GIVEN_TYPE_IX e_type_dying_animation
    jr z, not_dying_animation_no_gravity

    ;;Check if die was the last state
    ld a, e_last_anim_state(ix)
    IS_EQUALS 2
    ret z
    ld e_last_anim_state(ix), #2

    call player_no_gravity_die
    ret

    not_dying_animation_no_gravity:

    ;;Check if player_no_gravity is moving
    ld a, e_vel_x(ix)
    IS_EQUALS 0
    jr nz, is_moving_no_gravity

    ;;is_not_moving:
    ;;Check if idle was the last state
    ld a, e_last_anim_state(ix)
    IS_EQUALS 0
    ret z
    ld e_last_anim_state(ix), #0

    call player_no_gravity_idle
    ret

is_moving_no_gravity:
    ;;Check if run was the last state
    ld a, e_last_anim_state(ix)
    IS_EQUALS 1
    jr nz, execute_anim_run_no_gravity

    ;;Same state == last_state. Check if different facing right
    ld a, e_last_facing_right(ix)
    ld b, e_facing_right(ix)
    IS_EQUALS_REGISTER b
    ret z

    execute_anim_run_no_gravity:
    ld e_last_anim_state(ix), #1
    ld a, e_facing_right(ix)
    ld e_last_facing_right(ix), a
    call player_no_gravity_run
    ret

ret